/**
 * Encryption reference:
 *
 * - https://github.com/darknessomi/musicbox/wiki
 * - https://github.com/Binaryify/NeteaseCloudMusicApi/blob/master/util/crypto.js
 * - https://github.com/stkevintan/Cube/blob/master/src/model/Crypto.js
 */

const crypto = require('crypto')
const bigInt = require('big-integer')

const modulus = '00e0b509f6259df8642dbc35662901477df22677ec152b5ff68ace615bb7b725152b3ab17a876aea8a5aa76d2e417629ec4ee341f56135fccf695280104e0312ecbda92557c93870114af6c9d05c4f7f0c3685b7a46bee255932575cce10b424d813cfe4875d3e82047b97ddef52741d546b8e289dc6935b3ece0462db0a22b8e7'
const nonce = '0CoJUm6Qyw8W8jud'
const pubKey = '010001'
const table = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'
const iv = '0102030405060708'

function createSecretKey(len) {
  const chars = []
  for (let i = 0; i < len; i++) {
    let idx = Math.random() * table.length >> 0
    chars.push(table[idx])
  }
  return chars.join('')
}

function addPadding(text, size) {
  let prefix = ''
  for (let i = 0; i < size - text.length; i++) {
    prefix += '0'
  }
  return prefix + text
}

function aesEncrypt(text, secKey) {
  const cipher = crypto.createCipheriv('aes-128-cbc', secKey, iv)
  let encrypted = cipher.update(text, 'utf8', 'base64')
  encrypted += cipher.final('base64')
  return encrypted
}

function rsaEncrypt(text, pubKey, modulus) {
  text = text.split('').reverse().join('')
  const biText = bigInt(Buffer.from(text).toString('hex'), 16)
  const biEx = bigInt(pubKey, 16)
  const biMod = bigInt(modulus, 16)
  const biRet = biText.modPow(biEx, biMod)
  return addPadding(biRet.toString(16), 256)
}

function encrypt(obj) {
  const text = JSON.stringify(obj)
  const secKey = createSecretKey(16)
  const params = aesEncrypt(aesEncrypt(text, nonce), secKey)
  const encSecKey = rsaEncrypt(secKey, pubKey, modulus)
  return { params, encSecKey }
}

module.exports = encrypt
